@@ -1,12 +1,13 @@
 from collections import abc, deque
 from urllib.error import URLError
+from pathlib import Path
 
 from ..core.http.cached_request import CachedRequests
 from ..core.parser.soup import SoupParser
 from ..utils.urls import queryjoin, urljoin, filter_params
 from ..utils.module import find_function, filter_kwargs
 from .exceptions import *
-from .actions import UrlPatternAction, UrlRenderAction
+from .actions import UrlPatternAction, UrlRenderAction, CurlAction, FileAction
 from .reducer import ReducerMixin
 from .meta import ResponseMeta
 from .event import CRAWLING_STARTED, CRAWLING_COMPLETED, VISITING_URL, CRAWLING_STOPPED, catch_crawl_exception
@@ -24,10 +25,21 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
         self.results = {} if collect_results else None
 
     def _resolve_link(self, link, action, response=None):
+        if isinstance(action, FileAction):
+            if link.startswith('file:///'):
+                return link
+            else:
+                return Path(link).absolute().as_uri()
+
+        if isinstance(action, CurlAction):
+            curl = link.replace('\n', '')
+            return curl
+
         if isinstance(action, UrlRenderAction): 
             host = action.host or response.url
         else:
             host = response.url
+
         if isinstance(link, abc.Mapping):
             return queryjoin(host, link)
         return urljoin(host, link)
@@ -45,6 +57,7 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
                 links = [host]
             else:
                 raise URLError("urlrender: host or renderer must be specified!")
+            return links
         elif isinstance(action, UrlPatternAction):
             if urlpattern_renderer := action.urlpattern_renderer:
                 urlpattern = self.dispatch(
@@ -61,25 +74,47 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
                 'urlpattern': urlpattern
             }
             links = self.parse_linkpattern(response.content, **kwargs)
-        return links
+            return links
+        elif isinstance(action, CurlAction):
+            if curlrenderer:= action.curlrenderer:
+                curls = self.dispatch(
+                    'curlrenderer', curlrenderer,
+                    curl=action.curl, parent_resposne=response, responsemap=responsemap, context=context
+                )
+            elif curl := action.curl:
+                curls = [curl]
+            return curls
+        elif isinstance(action, FileAction):
+            if pathrenderer := action.pathrenderer:
+                paths = self.dispatch(
+                    'pathrenderer', pathrenderer,
+                    path=action.path, parent_resposne=response, responsemap=responsemap, context=context
+                )
+            elif path := action.path:
+                paths = [path]
+            return paths
+        else:
+            raise CannotFindAction(f'{action} is not memeber of action')
+                
     
     def _dispatch_fields(self, action, url):
+        if not hasattr(action, 'fields'):
+            return url
         url = filter_params(url, action.fields)
         return url
     
     def _dispatch_headers(self, action):
         headers = self.get_headers()
-        headers.update(action.headers or {})
+        if hasattr(action, 'headers'):
+            headers.update(action.headers or {})
         return headers
     
     def _dispatch_cookies(self, action):
         cookies = self.get_cookies()
-        cookies.update(action.cookies or {})
+        if hasattr(action, 'cookies'):
+            cookies.update(action.cookies or {})
         return cookies
     
-    def _dispatch_refresh(self, action):
-        return action.refresh
-
     def _dispatch_extractor(self, action, meta, response, context):
         extractset = {}
         if module := action.extractor:
@@ -103,14 +138,18 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
 
 
     def _dispatch_urlfilter(self, action, url, responsemap, context):
+        if not hasattr(action, 'urlfiler'):
+            return True
+
         return self.dispatch(
             'urlfilter', action.urlfilter,
             url=url, responsemap=responsemap,  context=context
         )
     
-    
     def _dispatch_payloader(self, action, context):
-        if not action.payloader:
+        if not hasattr(action, 'payloader'):
+            yield  
+        elif not action.payloader:
             yield
         else:
             payloads = self.dispatch('payloader', action.payloader, context=context)
@@ -119,23 +158,20 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
             else:
                 yield from payloads
         
-
-    def _dispatch_response(self, url, payloads, refresh, method, **kwargs):
-        if m:= method:
-            m = m.lower()
-        else:
-            m = 'post' if payloads else 'get'
-
-        if m == 'get':
-            response = self.get(url, refresh=refresh, **kwargs)
-        else:
-            response = self.post(url, data=payloads, **kwargs)
-            
-        return response
+    def _dispatch_response(self, action, **kwargs):
+        if isinstance(action, CurlAction):
+            return self.from_curl(
+                kwargs.get('url'), action.refresh, self.get_delay(action.delay)
+            )
+        method = action.method or ('post' if kwargs.get('data') else 'get')
+        return self.fetch(method, action.refresh, self.get_delay(action.delay), **kwargs)
 
 
     def _dispatch_referer(self, action, response):
         headers = self.get_headers()
+        if not hasattr(action, 'referer'):
+            return headers
+
         if action.referer and response:
             try:
                 referer = response.crawler.responsemap[action.referer]
@@ -175,14 +211,14 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
 
         while response_queue:
             response = response_queue.pop()
-
-            refresh = self._dispatch_refresh(action)
+            
             cookies = self._dispatch_cookies(action)
             headers = self._dispatch_headers(action)
             headers.update(self._dispatch_referer(action, response))
 
             is_parsable = True
-            for link in self._dispatch_renderer(action, response, _responsemap, context):
+            for link in self._dispatch_renderer(action, response, _responsemap, context):                    
+
                 url = self._resolve_link(link, action, response)
                 url = self._dispatch_fields(action, url)
 
@@ -197,8 +233,9 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
 
                 #check post method
                 for payloads in self._dispatch_payloader(action, context):
-                    sub_response = self._dispatch_response(url, payloads, refresh, action.method, headers=headers, cookies=cookies, delay=action.delay)
-
+                    sub_response = self._dispatch_response(
+                        action, url=url, data=payloads, headers=headers, cookies=cookies
+                    )
                     ## listen visiting url
                     _visite_count += 1
                     self.crawl_listener(module_name, VISITING_URL, {'visite_count': _visite_count})
