@@ -12,7 +12,7 @@ COUNT_PER_PAGE = 100
 config = {
     'REQUEST_CACHE_BACKEND': 'sqlite',
     # 'REQUEST_CACHE_BACKEND': 'memory',
-    'REQUEST_DELAY': (1, 1.5),
+    'REQUEST_DELAY': (2, 3),
     'REQUESTS_TIMEOUT': 5,
     'REQUEST_CACHE_CACHE_NAME': 'naverplace.sqlite'
 
@@ -39,7 +39,7 @@ class NaverPlaceCrawler(BaseCrawler):
         ),
         urlrender(
             'https://pcmap-api.place.naver.com/graphql',
-            payloader='naverplace_review_payloader',
+            payloader='naverplace_review_payloader', refresh=True,
             parser='naverplace_review_parser', name='naverplace_review'
         ),
     ]
@@ -115,10 +115,6 @@ class NaverPlaceCrawler(BaseCrawler):
 
 
 
-
-
-
-
     
 def crawl_naver_place(place_name):
     npc = NaverPlaceCrawler(place_name)
